@@ -14,11 +14,48 @@
 
 from datetime import datetime
 
-from cinderclient.tests.unit import fakes
 from cinderclient.v3 import client
+
+from cinderclient.tests.unit import fakes
 from cinderclient.tests.unit.v2 import fakes as fake_v2
 
 
+fake_attachment = {'attachment': {
+    'status': 'reserved',
+    'detached_at': '',
+    'connection_info': {},
+    'attached_at': '',
+    'attach_mode': None,
+    'id': 'a232e9ae',
+    'instance': 'e84fda45-4de4-4ce4-8f39-fc9d3b0aa05e',
+    'volume_id': '557ad76c-ce54-40a3-9e91-c40d21665cc3', }}
+
+fake_connection_info = {
+    'auth_password': 'i6h9E5HQqSkcGX3H',
+    'attachment_id': 'a232e9ae',
+    'target_discovered': False,
+    'encrypted': False,
+    'driver_volume_type': 'iscsi',
+    'qos_specs': None,
+    'target_iqn': 'iqn.2010-10.org.openstack:volume-557ad76c',
+    'target_portal': '10.117.36.28:3260',
+    'volume_id': '557ad76c-ce54-40a3-9e91-c40d21665cc3',
+    'target_lun': 0,
+    'access_mode': 'rw',
+    'auth_username': 'MwRrnAFLHN7enw5R95yM',
+    'auth_method': 'CHAP'}
+
+fake_connector = {
+    'initiator': 'iqn.1993-08.org.debian:01:b79dbce99387',
+    'mount_device': '/dev/vdb',
+    'ip': '10.117.36.28',
+    'platform': 'x86_64',
+    'host': 'os-2',
+    'do_local_attach': False,
+    'os_type': 'linux2',
+    'multipath': False}
+
+
 def _stub_group(detailed=True, **kwargs):
     group = {
         "name": "test-1",
@@ -77,8 +114,9 @@ class FakeClient(fakes.FakeClient, client.Client):
                                'project_id', 'auth_url',
                                extensions=kwargs.get('extensions'))
         self.api_version = api_version
+        global_id = "req-f551871a-4950-4225-9b2c-29a14c8f075e"
         self.client = FakeHTTPClient(api_version=api_version,
-                                     **kwargs)
+                global_request_id=global_id, **kwargs)
 
     def get_volume_api_version_from_endpoint(self):
         return self.client.get_volume_api_version_from_endpoint()
@@ -107,6 +145,7 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
                 'state': 'up',
                 'updated_at': datetime(2012, 10, 29, 13, 42, 2),
                 'cluster': 'cluster1',
+                'backend_state': 'up',
             },
             {
                 'id': 2,
@@ -117,6 +156,7 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
                 'state': 'down',
                 'updated_at': datetime(2012, 9, 18, 8, 3, 38),
                 'cluster': 'cluster1',
+                'backend_state': 'down',
             },
             {
                 'id': 3,
@@ -136,6 +176,11 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         if not self.api_version.matches('3.7'):
             for svc in services:
                 del svc['cluster']
+
+        if not self.api_version.matches('3.49'):
+            for svc in services:
+                if svc['binary'] == 'cinder-volume':
+                    del svc['backend_state']
         return (200, {}, {'services': services})
 
     #
@@ -239,12 +284,9 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
     #
     # Attachments
     #
+
     def post_attachments(self, **kw):
-        return (202, {}, {
-            'attachment': {'instance': 1234,
-                           'name': 'attachment-1',
-                           'volume_id': 'fake_volume_1',
-                           'status': 'reserved'}})
+        return (200, {}, fake_attachment)
 
     def get_attachments(self, **kw):
         return (200, {}, {
@@ -257,6 +299,16 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
                              'volume_id': 'fake_volume_2',
                              'status': 'reserverd'}]})
 
+    def post_attachments_a232e9ae_action(self, **kw):  # noqa: E501
+        attached_fake = fake_attachment
+        attached_fake['status'] = 'attached'
+        return (200, {}, attached_fake)
+
+    def post_attachments_1234_action(self, **kw):  # noqa: E501
+        attached_fake = fake_attachment
+        attached_fake['status'] = 'attached'
+        return (200, {}, attached_fake)
+
     def get_attachments_1234(self, **kw):
         return (200, {}, {
             'attachment': {'instance': 1234,
@@ -292,7 +344,7 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         return (200, {}, {'group_type': {'id': 1,
                           'name': 'test-type-1',
                           'description': 'test_type-1-desc',
-                          'group_specs': {u'key': u'value'}}})
+                          'group_specs': {'key': 'value'}}})
 
     def get_group_types_2(self, **kw):
         return (200, {}, {'group_type': {'id': 2,
@@ -369,6 +421,10 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         action = list(body)[0]
         if action == 'delete':
             assert 'delete-volumes' in body[action]
+        elif action in ('enable_replication', 'disable_replication',
+                        'failover_replication', 'list_replication_targets',
+                        'reset_status'):
+            assert action in body
         else:
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, {})
@@ -413,9 +469,6 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
     def put_group_snapshots_1234(self, **kw):
         return (200, {}, {'group_snapshot': {}})
 
-    def post_groups_1234_action(self, **kw):
-        return (202, {}, {})
-
     def get_groups_5678(self, **kw):
         return (200, {}, {'group':
                           _stub_group(id='5678')})
@@ -441,9 +494,6 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
     def post_group_snapshots_5678_action(self, **kw):
         return (202, {}, {})
 
-    def get_group_snapshots_5678(self, **kw):
-        return (200, {}, {'group_snapshot': _stub_group_snapshot(id='5678')})
-
     def delete_group_snapshots_1234(self, **kw):
         return (202, {}, {})
 
@@ -544,12 +594,81 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         }
         return 200, {}, {'message': message}
 
+    def put_os_services_set_log(self, body):
+        return (202, {}, {})
+
+    def put_os_services_get_log(self, body):
+        levels = [{'binary': 'cinder-api', 'host': 'host1',
+                   'levels': {'prefix1': 'DEBUG', 'prefix2': 'INFO'}},
+                  {'binary': 'cinder-volume', 'host': 'host@backend#pool',
+                   'levels': {'prefix3': 'WARNING', 'prefix4': 'ERROR'}}]
+        return (200, {}, {'log_levels': levels})
+
+    def get_volumes_summary(self, **kw):
+        return 200, {}, {"volume-summary": {'total_size': 5,
+                                            'total_count': 5,
+                                            'metadata': {
+                                                "test_key": ["test_value"]
+                                            }
+                                            }
+                         }
+
+    def post_workers_cleanup(self, **kw):
+        response = {
+            'cleaning': [{'id': '1', 'cluster_name': 'cluster1',
+                          'host': 'host1', 'binary': 'binary'},
+                         {'id': '3', 'cluster_name': 'cluster1',
+                          'host': 'host3', 'binary': 'binary'}],
+            'unavailable': [{'id': '2', 'cluster_name': 'cluster2',
+                             'host': 'host2', 'binary': 'binary'}],
+        }
+        return 200, {}, response
+
     #
     # resource filters
     #
     def get_resource_filters(self, **kw):
         return 200, {}, {'resource_filters': []}
 
+    def get_volume_transfers_detail(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        transfer1 = '5678'
+        transfer2 = 'f625ec3e-13dd-4498-a22a-50afd534cc41'
+        return (200, {},
+                {'transfers': [
+                    fake_v2._stub_transfer_full(transfer1, base_uri,
+                                                tenant_id),
+                    fake_v2._stub_transfer_full(transfer2, base_uri,
+                                                tenant_id)]})
+
+    def get_volume_transfers_5678(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        transfer1 = '5678'
+        return (200, {},
+                {'transfer':
+                 fake_v2._stub_transfer_full(transfer1, base_uri, tenant_id)})
+
+    def delete_volume_transfers_5678(self, **kw):
+        return (202, {}, None)
+
+    def post_volume_transfers(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        transfer1 = '5678'
+        return (202, {},
+                {'transfer': fake_v2._stub_transfer(transfer1, base_uri,
+                                                    tenant_id)})
+
+    def post_volume_transfers_5678_accept(self, **kw):
+        base_uri = 'http://localhost:8776'
+        tenant_id = '0fa851f6668144cf9cd8c8419c1646c1'
+        transfer1 = '5678'
+        return (200, {},
+                {'transfer': fake_v2._stub_transfer(transfer1, base_uri,
+                                                    tenant_id)})
+
 
 def fake_request_get():
     versions = {'versions': [{'id': 'v1.0',
@@ -589,3 +708,31 @@ def fake_request_get():
                               'updated': '2016-02-08T12:20:21Z',
                               'version': '3.16'}]}
     return versions
+
+
+def fake_request_get_no_v3():
+    versions = {'versions': [{'id': 'v1.0',
+                              'links': [{'href': 'http://docs.openstack.org/',
+                                         'rel': 'describedby',
+                                         'type': 'text/html'},
+                                        {'href': 'http://192.168.122.197/v1/',
+                                         'rel': 'self'}],
+                              'media-types': [{'base': 'application/json',
+                                               'type': 'application/'}],
+                              'min_version': '',
+                              'status': 'DEPRECATED',
+                              'updated': '2016-05-02T20:25:19Z',
+                              'version': ''},
+                             {'id': 'v2.0',
+                              'links': [{'href': 'http://docs.openstack.org/',
+                                         'rel': 'describedby',
+                                         'type': 'text/html'},
+                                        {'href': 'http://192.168.122.197/v2/',
+                                         'rel': 'self'}],
+                              'media-types': [{'base': 'application/json',
+                                               'type': 'application/'}],
+                              'min_version': '',
+                              'status': 'SUPPORTED',
+                              'updated': '2014-06-28T12:20:21Z',
+                              'version': ''}]}
+    return versions
