@@ -324,6 +324,49 @@ class ShellTest(utils.TestCase):
         self.assertTrue(mock_find_volume.called)
         self.assert_called('POST', '/attachments', body=expected)
 
+    @ddt.data({'cmd': '1234 1233',
+               'body': {'instance_uuid': '1233',
+                        'connector': {},
+                        'volume_uuid': '1234',
+                        'mode': 'ro'}},
+              {'cmd': '1234 1233 '
+                      '--connect True '
+                      '--ip 10.23.12.23 --host server01 '
+                      '--platform x86_xx '
+                      '--ostype 123 '
+                      '--multipath true '
+                      '--mountpoint /123 '
+                      '--initiator aabbccdd',
+               'body': {'instance_uuid': '1233',
+                        'connector': {'ip': '10.23.12.23',
+                                      'host': 'server01',
+                                      'os_type': '123',
+                                      'multipath': 'true',
+                                      'mountpoint': '/123',
+                                      'initiator': 'aabbccdd',
+                                      'platform': 'x86_xx'},
+                        'volume_uuid': '1234',
+                        'mode': 'ro'}},
+              {'cmd': 'abc 1233',
+               'body': {'instance_uuid': '1233',
+                        'connector': {},
+                        'volume_uuid': '1234',
+                        'mode': 'ro'}})
+    @mock.patch('cinderclient.utils.find_resource')
+    @ddt.unpack
+    def test_attachment_create_with_mode(self, mock_find_volume, cmd, body):
+        mock_find_volume.return_value = volumes.Volume(self,
+                                                       {'id': '1234'},
+                                                       loaded=True)
+        command = ('--os-volume-api-version 3.54 '
+                   'attachment-create '
+                   '--mode ro ')
+        command += cmd
+        self.run_command(command)
+        expected = {'attachment': body}
+        self.assertTrue(mock_find_volume.called)
+        self.assert_called('POST', '/attachments', body=expected)
+
     @mock.patch.object(volumes.VolumeManager, 'findall')
     def test_attachment_create_duplicate_name_vol(self, mock_findall):
         found = [volumes.Volume(self, {'id': '7654', 'name': 'abc'},
@@ -549,13 +592,8 @@ class ShellTest(utils.TestCase):
                          '--volume-type 4321 1')
         self.assert_called('GET', '/volumes/1234')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
                                'size': 1,
-                               'user_id': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'group_id': '5678',
@@ -578,13 +616,8 @@ class ShellTest(utils.TestCase):
         self.run_command(cmd)
         self.assert_called('GET', '/volumes/1234')
         expected = {'volume': {'imageRef': None,
-                               'project_id': None,
-                               'status': 'creating',
-                               'user_id': None,
                                'size': None,
                                'availability_zone': None,
-                               'source_replica': None,
-                               'attach_status': 'detached',
                                'source_volid': None,
                                'consistencygroup_id': None,
                                'name': None,
@@ -1299,3 +1332,19 @@ class ShellTest(utils.TestCase):
                     'service_id': 1}
 
         self.assert_called('POST', '/workers/cleanup', body=expected)
+
+    def test_create_transfer(self):
+        self.run_command('transfer-create 1234')
+        expected = {'transfer': {'volume_id': 1234,
+                                 'name': None,
+                                 }}
+        self.assert_called('POST', '/volume-transfers', body=expected)
+
+    def test_create_transfer_no_snaps(self):
+        self.run_command('--os-volume-api-version 3.55 transfer-create '
+                         '--no-snapshots 1234')
+        expected = {'transfer': {'volume_id': 1234,
+                                 'name': None,
+                                 'no_snapshots': True
+                                 }}
+        self.assert_called('POST', '/volume-transfers', body=expected)
